Attribute VB_Name = "Tools"
'  FILE:       Tools.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    This module contains functions to clear run history, create
'              a new workspace definition file and import and export
'              workspaces. (Tools menu on frmMain)
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit
Private Const mstrModuleName As String = "Tools."

Private dbNew As Database
Private dbOld As Database

' Used by the Delete Version information procedure
Private Enum QDefType
    iStep = 0
    iIterator
    iConstraint
    iConsGlobal
End Enum
Private Const iShift As Integer = 4 'last element of QDefType + 1

Public Sub ClearHistory()
    
    Dim recWorkspaces As Recordset
    Dim qryWsp As QueryDef

    On Error GoTo ClearHistoryErr
    
    If dbsAttTool Is Nothing Then Exit Sub
    
    If Not SaveDB Then Exit Sub
    
    ShowBusy
    
    Set dbNew = dbsAttTool
    
    ' Delete details of all previous runs from the database
    Call DeleteRunHistory(dbNew)
    
    Call ReadWorkspaces(dbNew, recWorkspaces, qryWsp, bSelectArchivedRecords:=True)
    
    If recWorkspaces.RecordCount <> 0 Then
        Do
            ' Delete version information for all steps and related entities
            ' (constraints and iterators)
            Call DeleteVersionHistory(recWorkspaces!workspace_id, recWorkspaces!archived_flag)
            recWorkspaces.MoveNext
            
        Loop Until recWorkspaces.EOF
    End If
    recWorkspaces.Close
    qryWsp.Close
    
    Set dbNew = Nothing
    ShowFree
    
    Exit Sub
    
ClearHistoryErr:
    ShowFree
    LogErrors Errors
    ShowError errClearHistoryFailed
    
End Sub

Public Sub CreateDB()
    Dim sCreateDB As String
    Dim iTemp As Integer

    On Error GoTo CreateDBErr
    
    ' Prompt for the name of the workspace definition file to be created
    sCreateDB = BrowseDBFileNew
    If StringEmpty(sCreateDB) Then Exit Sub
    
    ShowBusy
    
    If InstrR(sCreateDB, gsExtSeparator) = 0 Then sCreateDB = sCreateDB & gsDefDBFileExt

    Set dbNew = wrkJet.CreateDatabase(sCreateDB, dbLangGeneral)
    BugAssert Not dbNew Is Nothing

    Call CreateTables
    
    Call InsertDbDetails(gsVersion)
    
    Call CloseDB(dbNew)
    ShowFree
    
    Exit Sub
    
CreateDBErr:
    ShowFree
    LogErrors Errors
    ShowError errCreateDBFailed
    Call CloseDB(dbNew)
    
End Sub
Private Sub DeleteWspDetails(lWspId As Long)
    ' Delete all workspace history records from the database for archived workspaces

    Dim sDelete(0 To 2) As String
    Dim qyDelete As DAO.QueryDef
    Dim iTemp As Integer
    
    On Error GoTo DeleteWspDetailsErr
    
    sDelete(0) = "workspace_parameters"
    sDelete(1) = "workspace_connections"
    sDelete(2) = "att_workspaces"
    
    For iTemp = LBound(sDelete) To UBound(sDelete)
        Set qyDelete = dbNew.CreateQueryDef("", "delete from " & sDelete(iTemp) & _
                " where workspace_id = [w_id] ")
        Call ExecuteQDef(qyDelete, lWspId)
        qyDelete.Close
    Next iTemp
    
    Exit Sub

DeleteWspDetailsErr:
    LogErrors Errors
    Err.Raise vbObjectError + errDeleteDBRecordFailed, mstrModuleName & "DeleteWspDetails", _
            LoadResString(errDeleteDBRecordFailed)
    
End Sub
Private Sub DeleteVersionHistory(lWspId As Long, bArchived As Boolean)
    ' Deletes version information for the steps in the passed in workspace
    ' if bArchived is True then delete all records in the workspace
    
    Dim qy As DAO.QueryDef
    Dim rSteps As DAO.Recordset
    Dim sTableName(iStep To iConstraint) As String
    Dim qyDelete(iStep To iConsGlobal + iShift) As DAO.QueryDef
    Dim iTemp As QDefType
    Dim iIndex As Integer
    
    On Error GoTo DeleteVersionHistoryErr

    Call ReadStepsInWorkspace(rSteps, qy, lWspId, dbLoad:=dbNew, _
            bSelectArchivedRecords:=True)
    
    If rSteps.RecordCount = 0 Then
        rSteps.Close
        qy.Close
        Exit Sub
    End If
    
    sTableName(iStep) = "att_steps"
    sTableName(iIterator) = "iterator_values"
    sTableName(iConstraint) = "step_constraints"
    
    For iTemp = iStep To iConstraint
        Set qyDelete(iTemp) = dbNew.CreateQueryDef("", _
                "delete from " & sTableName(iTemp) & _
                " where step_id = [st_id] ")
        Set qyDelete(iTemp + iShift) = dbNew.CreateQueryDef("", qyDelete(iTemp).SQL & _
                " and version_no <> [ver_no] ")
    Next iTemp
    
    Set qyDelete(iConsGlobal) = dbNew.CreateQueryDef("", _
            "delete from " & sTableName(iConstraint) & _
            " where global_step_id = [st_id] ")
    Set qyDelete(iConsGlobal + iShift) = dbNew.CreateQueryDef("", _
            qyDelete(iConsGlobal).SQL & " and global_version_no <> [ver_no] ")

    rSteps.MoveFirst
    
    While Not rSteps.EOF
        For iTemp = iStep To iIterator
            If bArchived Or rSteps![archived_flag] Then
                iIndex = iTemp
            Else: iIndex = iTemp + iShift
            End If
            
            ' Delete all archived step and iterator records for the step
            Call ExecuteQDef(qyDelete(iIndex), rSteps![step_id], rSteps![version_no])
        Next iTemp
            
        iIndex = IIf(rSteps![global_flag], iConsGlobal, iConstraint)
        If Not bArchived And Not rSteps![archived_flag] Then iIndex = iIndex + iShift
        
        ' Delete all archived constraint records for the step
        Call ExecuteQDef(qyDelete(iIndex), rSteps![step_id], rSteps![version_no])
        
        rSteps.MoveNext
    Wend
    
    For iTemp = iStep To iConsGlobal
        qyDelete(iTemp).Close
        qyDelete(iTemp + iShift).Close
    Next iTemp
    
    If bArchived Then
        Call DeleteWspDetails(lWspId)
    End If
    
    rSteps.Close
    qy.Close
    
    Exit Sub

DeleteVersionHistoryErr:
    LogErrors Errors
    Err.Raise vbObjectError + errDeleteDBRecordFailed, mstrModuleName & "DeleteVersionHistory", _
            LoadResString(errDeleteDBRecordFailed)
    
End Sub
Private Sub ExecuteQDef(qy As DAO.QueryDef, lId As Long, Optional sVersion As String)
    ' Initializes the step_id and version parameters of the QueryDef object and
    ' executes it

    On Error GoTo ExecuteQDefErr
    
    ' Initialize the parameter values
    qy.Parameters(0).Value = lId
    If qy.Parameters.Count > 1 Then qy.Parameters(1).Value = sVersion
    qy.Execute dbFailOnError
    
    Exit Sub

ExecuteQDefErr:
    LogErrors Errors
    Err.Raise vbObjectError + errDeleteDBRecordFailed, mstrModuleName & "ExecuteQDef", _
            LoadResString(errDeleteDBRecordFailed)
    
End Sub


Public Sub DoExport(lWsp As Long)

    On Error GoTo DoExportErr
    
    BugAssert Not dbNew Is Nothing
    BugAssert Not dbOld Is Nothing
    
    LoadWspData lWsp
    
    Exit Sub
    
DoExportErr:
    LogErrors Errors
    ShowError errExportWspFailed
    
End Sub
Public Sub DoImport(lWsp As Long)

    On Error GoTo DoImportErr
    
    BugAssert Not dbNew Is Nothing
    BugAssert Not dbOld Is Nothing
    
    LoadWspData lWsp
    
    Exit Sub
    
DoImportErr:
    LogErrors Errors
    ShowError errImportWspFailed
    
End Sub

Public Sub ImportWsp()

    Dim sImportFrom  As String
    
    On Error GoTo ImportWspErr
    
    If dbsAttTool Is Nothing Then Exit Sub
    
    If Not SaveDB Then Exit Sub
    
    sImportFrom = BrowseImportFile
    
    ' Ensure that we are not importing from the database that is currently open
    If Not StringEmpty(sImportFrom) And sImportFrom <> dbsAttTool.Name Then
        Set dbOld = wrkJet.OpenDatabase(sImportFrom)
        Set dbNew = dbsAttTool
    
        Call ShowWorkspaceImport(dbOld)
        
        CloseDB dbOld
        
        Set dbNew = Nothing
    End If
    
    Exit Sub
    
ImportWspErr:
    LogErrors Errors
    ShowError errImportWspFailed
    
End Sub
Public Sub ExportWsp()

    Dim sExportTo  As String
    
    On Error GoTo ExportWspErr
    
    If dbsAttTool Is Nothing Then Exit Sub
    
    If Not SaveDB Then Exit Sub
    
    sExportTo = BrowseExportFile
    
    ' Ensure that we are not Exporting from the database that is currently open
    If Not StringEmpty(sExportTo) And sExportTo <> dbsAttTool.Name Then
        Set dbNew = wrkJet.OpenDatabase(sExportTo)
        Set dbOld = dbsAttTool
    
        Call ShowWorkspaceExport(dbOld)
        
        CloseDB dbNew
        
        Set dbOld = Nothing
    End If
    
    Exit Sub
    
ExportWspErr:
    LogErrors Errors
    ShowError errExportWspFailed
    
End Sub

Private Function SaveDB() As Boolean

    ' Saves all open workspaces
    Dim nodWspNode As Node
    Dim nodNextNode As Node
    
    On Error GoTo SaveDBErr
    
    SaveDB = False

    ' Make sure that the user really wants to save changes
    If Not Confirm(conSaveDB, contSaveDB, Buttons:=vbYesNo) Then
        Exit Function
    End If
    
    Set nodWspNode = GetRootNode(frmMain.treStepView)
    
    Do While Not nodWspNode Is Nothing
        If nodWspNode.Key <> nodWspNode.LastSibling.Key Then
            Set nodNextNode = nodWspNode.Next
        Else
            Set nodNextNode = Nothing
        End If
        
        Call SaveWorkspace(MakeIdentifierValid(nodWspNode.Key))
        Set nodWspNode = nodNextNode
    Loop
    
    SaveDB = True
    Exit Function
    
SaveDBErr:
    LogErrors Errors
    Err.Raise vbObjectError + errProgramError, mstrModuleName & "SaveDB", _
            LoadResString(errProgramError)

End Function
Private Sub CreateTables()
    Dim iTemp As Integer
    Dim qy As DAO.QueryDef
    
    On Error GoTo CreateTablesErr
    
    Call InitCreateSQLArray
    
    For iTemp = LBound(sCreateTables) To UBound(sCreateTables)

        Set qy = dbNew.CreateQueryDef("", sCreateTables(iTemp))
        qy.Execute dbFailOnError
        qy.Close
    Next iTemp
    
    ReDim sCreateTables(0)

    Exit Sub

CreateTablesErr:
    LogErrors Errors
    Err.Raise vbObjectError + errUnableToCreateNewObject, mstrModuleName & "CreateTables", _
            LoadResString(errUnableToCreateNewObject)

End Sub
Private Sub CloseDB(dbClose As Database)
    
    On Error GoTo CloseDBErr
    
    dbClose.Close
    Set dbClose = Nothing

    Exit Sub

CloseDBErr:
    LogErrors Errors
    Resume Next

End Sub

Private Sub InsertStep(rSteps As DAO.Recordset, cIds As cStepIds, lNewWspId As Long)

    Dim sInsert As String
    Dim cTempStr As New cStringSM
    
    On Error GoTo InsertStepErr
    
    ' Create a temporary querydef object
    sInsert = "insert into att_steps " & _
        "( workspace_id, step_id, version_no, " & _
        " step_label, step_file_name, step_text, start_directory, " & _
        " parent_step_id, parent_version_no, sequence_no, " & _
        " enabled_flag, step_level, " & _
        " degree_parallelism, execution_mechanism, failure_details, " & _
        " continuation_criteria, global_flag, " & _
        " archived_flag, " & _
        " output_file_name, error_file_name, " & _
        " iterator_name ) values ( "

    ' log_file_name,
    
    sInsert = sInsert & Str(lNewWspId) & ", " & cIds.NewId(rSteps!step_id) & _
        ", " & cTempStr.MakeStringFieldValid(gstrMinVersion)

    sInsert = sInsert & ", " & cTempStr.MakeStringFieldValid("" & rSteps!step_label)
    sInsert = sInsert & ", " & cTempStr.MakeStringFieldValid("" & rSteps!step_file_name)
    sInsert = sInsert & ", " & cTempStr.MakeStringFieldValid("" & rSteps!step_text)
    sInsert = sInsert & ", " & cTempStr.MakeStringFieldValid("" & rSteps!start_directory)

    sInsert = sInsert & ", " & cIds.NewId(rSteps!parent_step_id) & _
        ", " & cTempStr.MakeStringFieldValid(gstrMinVersion) & _
        ", " & Str(rSteps!sequence_no) & ", " & Str(rSteps!enabled_flag) & ", " & Str(rSteps!step_level)

    sInsert = sInsert & ", " & cTempStr.MakeStringFieldValid("" & rSteps!degree_parallelism)
    sInsert = sInsert & ", " & Str(rSteps!execution_mechanism)

    sInsert = sInsert & ", " & cTempStr.MakeStringFieldValid("" & rSteps!failure_details) & _
        ", " & Str(rSteps!continuation_criteria) & _
        ", " & Str(rSteps!global_flag) & _
        ", " & Str(rSteps!archived_flag)

    sInsert = sInsert & ", " & cTempStr.MakeStringFieldValid("" & rSteps!output_file_name)
'    sInsert = sInsert & ", " & cTempStr.MakeStringFieldValid("" & rSteps!log_file_name)
    sInsert = sInsert & ", " & cTempStr.MakeStringFieldValid("" & rSteps!error_file_name)
    sInsert = sInsert & ", " & cTempStr.MakeStringFieldValid("" & rSteps!iterator_name)

    sInsert = sInsert & " ) "

    dbNew.Execute sInsert, dbFailOnError
    
    Exit Sub

InsertStepErr:
    LogErrors Errors
    Err.Raise vbObjectError + errInsertFailed, mstrModuleName & "InsertStep", _
            LoadResString(errInsertFailed)

End Sub
Private Sub InsertConstraint(rConstraints As DAO.Recordset, cIds As cStepIds, lConsId As Long)

    Dim sInsert As String
    Dim cTempStr As New cStringSM
    
    On Error GoTo InsertConstraintErr
    
    sInsert = "insert into step_constraints " & _
        "( constraint_id, step_id, version_no, " & _
        " constraint_type, global_step_id, global_version_no, sequence_no ) " & _
        " values ( " & _
        Str(lConsId) & ", " & cIds.NewId(rConstraints!step_id) & ", " & _
        cTempStr.MakeStringFieldValid(gstrMinVersion) & ", " & Str(rConstraints!constraint_type) & ", " & _
        cIds.NewId(rConstraints!global_step_id) & ", " & cTempStr.MakeStringFieldValid(gstrMinVersion) & ", " & _
        Str(rConstraints!sequence_no) & " ) "

    dbNew.Execute sInsert, dbFailOnError

    Exit Sub

InsertConstraintErr:
    LogErrors Errors
    Err.Raise vbObjectError + errInsertFailed, mstrModuleName & "InsertConstraint", _
            LoadResString(errInsertFailed)

End Sub
Private Sub InsertDbDetails(sDbDtls As String)

    Dim sInsert As String
    Dim cTemp As New cStringSM
    
    On Error GoTo InsertDbDetailsErr
    
    sInsert = "insert into db_details " & _
            "( db_version ) values ( " & cTemp.MakeStringFieldValid(sDbDtls) & " ) "

    dbNew.Execute sInsert, dbFailOnError
    
    Exit Sub

InsertDbDetailsErr:
    LogErrors Errors
    Err.Raise vbObjectError + errInsertFailed, mstrModuleName & "InsertDbDetails", _
            LoadResString(errInsertFailed)

End Sub
Private Sub InsertWorkspace(rWsp As DAO.Recordset, lWspId As Long)

    Dim sInsert As String
    Dim cTempStr As New cStringSM
    
    On Error GoTo InsertWorkspaceErr
    
    sInsert = "insert into att_workspaces " & _
            "( workspace_id, workspace_name, " & _
            " archived_flag ) " & _
            " values ( " & _
            Str(lWspId) & _
            ", " & cTempStr.MakeStringFieldValid("" & rWsp!workspace_name) & _
            ", " & Str(rWsp!archived_flag) & _
            " ) "
    dbNew.Execute sInsert, dbFailOnError
    
    Exit Sub

InsertWorkspaceErr:
    LogErrors Errors
    Err.Raise vbObjectError + errInsertFailed, mstrModuleName & "InsertWorkspace", _
            LoadResString(errInsertFailed)

End Sub
Private Sub InsertParameter(rParams As DAO.Recordset, lParamId As Long, lWspId As Long)

    Dim sInsert As String
    Dim cTempStr As New cStringSM
    
    On Error GoTo InsertParameterErr
    
    sInsert = "insert into workspace_parameters " & _
            "( workspace_id, parameter_id, " & _
            " parameter_name, parameter_value, " & _
            " description, parameter_type ) " & _
            " values ( " & _
            Str(lWspId) & ", " & Str(lParamId) & _
            ", " & cTempStr.MakeStringFieldValid("" & rParams!parameter_name) & _
            ", " & cTempStr.MakeStringFieldValid("" & rParams!parameter_value) & _
            ", " & cTempStr.MakeStringFieldValid("" & rParams!Description) & _
            ", " & Str(rParams!parameter_type) & _
            " ) "
    dbNew.Execute sInsert, dbFailOnError
    
    Exit Sub

InsertParameterErr:
    LogErrors Errors
    Err.Raise vbObjectError + errInsertFailed, mstrModuleName & "InsertParameter", _
            LoadResString(errInsertFailed)

End Sub

Private Sub InsertConnection(rConns As DAO.Recordset, lConnId As Long, lWspId As Long)

    Dim sInsert As String
    Dim cTempStr As New cStringSM
    
    On Error GoTo InsertConnectionErr
    
    sInsert = "insert into workspace_connections " & _
             "( workspace_id, connection_id, connection_name, " & _
             " connection_value, description, " & _
             " no_count_display, no_execute, parse_query_only, ANSI_quoted_identifiers, " & _
             " ANSI_nulls, show_query_plan, show_stats_time, show_stats_io, " & _
             " parse_odbc_msg_prefixes, row_count, tsql_batch_separator, query_time_out, " & _
             " server_language, character_translation, regional_settings ) " & _
            " values ( " & _
            Str(lWspId) & ", " & Str(lConnId) & _
            ", " & cTempStr.MakeStringFieldValid("" & rConns!connection_name) & _
            ", " & cTempStr.MakeStringFieldValid("" & rConns!connection_value) & _
            ", " & cTempStr.MakeStringFieldValid("" & rConns!Description) & _
            ", " & Str(rConns!no_count_display) & ", " & Str(rConns!no_execute) & _
            ", " & Str(rConns!parse_query_only) & ", " & Str(rConns!ANSI_quoted_identifiers) & _
            ", " & Str(rConns!ANSI_nulls) & ", " & Str(rConns!show_query_plan) & _
            ", " & Str(rConns!show_stats_time) & ", " & Str(rConns!show_stats_io) & _
            ", " & Str(rConns!parse_odbc_msg_prefixes) & ", " & Str(rConns!row_count) & _
            ", " & cTempStr.MakeStringFieldValid("" & rConns!tsql_batch_separator) & _
            ", " & Str(rConns!query_time_out) & _
            ", " & cTempStr.MakeStringFieldValid("" & rConns!server_language) & _
            ", " & Str(rConns!character_translation) & ", " & Str(rConns!regional_settings) & _
            " ) "
    dbNew.Execute sInsert, dbFailOnError
    
    Exit Sub

InsertConnectionErr:
    LogErrors Errors
    Err.Raise vbObjectError + errInsertFailed, mstrModuleName & "InsertConnection", _
            LoadResString(errInsertFailed)

End Sub
Private Sub InsertConnDtl(rConns As DAO.Recordset, lConnId As Long, lWspId As Long)

    Dim sInsert As String
    Dim cTempStr As New cStringSM
    
    On Error GoTo InsertConnDtlErr
    
    sInsert = "insert into " & TBL_CONNECTION_DTLS & _
             "( " & FLD_ID_WORKSPACE & _
             ", " & FLD_ID_CONN_NAME & _
             ", " & FLD_CONN_DTL_CONNECTION_NAME & _
             ", " & FLD_CONN_DTL_CONNECTION_STRING & _
             ", " & FLD_CONN_DTL_CONNECTION_TYPE & " ) " & _
            " values " & _
            " ( " & Str(lWspId) & _
            ", " & Str(lConnId) & _
            ", " & cTempStr.MakeStringFieldValid("" & rConns.Fields(FLD_CONN_DTL_CONNECTION_NAME)) & _
            ", " & cTempStr.MakeStringFieldValid("" & rConns.Fields(FLD_CONN_DTL_CONNECTION_STRING)) & _
            ", " & Str(rConns.Fields(FLD_CONN_DTL_CONNECTION_TYPE)) & _
            " ) "
    dbNew.Execute sInsert, dbFailOnError
    
    Exit Sub

InsertConnDtlErr:
    LogErrors Errors
    Err.Raise vbObjectError + errInsertFailed, mstrModuleName & "InsertConnDtl", _
            LoadResString(errInsertFailed)

End Sub

Private Sub InsertIterator(rIterators As DAO.Recordset, cIds As cStepIds)

    Dim sInsert As String
    Dim cTempStr As New cStringSM
    
    On Error GoTo InsertIteratorErr
    
    sInsert = "insert into iterator_values " & _
            "( step_id, version_no, type, " & _
            " iterator_value, sequence_no ) " & _
            " values ( " & cIds.NewId(rIterators!step_id) & _
            ", " & cTempStr.MakeStringFieldValid(gstrMinVersion) & _
            ", " & Str(rIterators!Type) & _
            ", " & cTempStr.MakeStringFieldValid("" & rIterators!iterator_value) & _
            ", " & Str(rIterators!sequence_no) & _
            " ) "

    dbNew.Execute sInsert, dbFailOnError
    
    Exit Sub

InsertIteratorErr:
    LogErrors Errors
    Err.Raise vbObjectError + errInsertFailed, mstrModuleName & "InsertIterator", _
            LoadResString(errInsertFailed)

End Sub
Private Sub LoadSteps(lWspId As Long, lNewWspId As Long)

    Dim qy As DAO.QueryDef
    Dim qyIterators As DAO.QueryDef
    Dim qyConstraints As DAO.QueryDef
    Dim sIterators As String
    Dim sConstraints As String
    Dim rSteps As DAO.Recordset
    Dim rIterators As DAO.Recordset
    Dim rConstraints As DAO.Recordset
    Dim cIds As cStepIds
    Dim cConsSeq As cSequence
    
    On Error GoTo LoadStepsErr
    
    Call ReadStepsInWorkspace(rSteps, qy, lWspId, dbLoad:=dbOld, _
            bSelectArchivedRecords:=False)
    
    If rSteps.RecordCount = 0 Then
        rSteps.Close
        qy.Close
        Exit Sub
    End If
    
    sIterators = "select * from iterator_values " & _
             "where step_id = [st_id] " & _
             "and version_no = [ver_no] "
    
    sConstraints = "select * from step_constraints a " & _
             "where global_step_id = [g_st_id] " & _
             "and global_version_no = [g_ver_no] " & _
            " AND ( a.step_id = 0 or ( cint( mid( a.version_no, 1, instr( a.version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) = " & _
            " ( select max( cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) ) " & _
            " from att_steps AS d " & _
            " WHERE a.step_id = d.step_id " & _
            " and d.archived_flag = [archived] ) " & _
            " AND cint( mid( a.version_no, instr( a.version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) + 1 ) ) = " & _
            " ( select max( cint( mid( version_no, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) + 1 ) ) ) " & _
            " from att_steps AS y " & _
            " Where a.step_id = y.step_id " & _
            " AND cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) = " & _
            " ( select max( cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) ) " & _
            " from att_steps AS c " & _
            " WHERE y.step_id = c.step_id " & _
            " and c.archived_flag = [archived] ) ) ) ) "

    Set qyIterators = dbOld.CreateQueryDef("", sIterators)
    Set qyConstraints = dbOld.CreateQueryDef("", sConstraints)
    
    rSteps.MoveFirst
    Set cIds = New cStepIds
    Set cIds.StepDB = dbNew
    
    While Not rSteps.EOF
        cIds.AddStepId rSteps!step_id
        rSteps.MoveNext
    Wend
    
    rSteps.MoveFirst
    While Not rSteps.EOF
        Call InsertStep(rSteps, cIds, lNewWspId)
        
        If (rSteps!global_flag) Then
            qyConstraints.Parameters!archived = False
            qyConstraints.Parameters!g_st_id = rSteps!step_id
            qyConstraints.Parameters!g_ver_no = rSteps!version_no
    
            Set rConstraints = qyConstraints.OpenRecordset(dbOpenSnapshot)
            
            If rConstraints.RecordCount <> 0 Then
                rConstraints.MoveFirst
                Set cConsSeq = New cSequence
                Set cConsSeq.IdDatabase = dbNew
                cConsSeq.IdentifierColumn = "constraint_id"
                
                While Not rConstraints.EOF
                    Call InsertConstraint(rConstraints, cIds, cConsSeq.Identifier)
                    rConstraints.MoveNext
                Wend
            End If
            rConstraints.Close
        Else
            qyIterators.Parameters!st_id = rSteps!step_id
            qyIterators.Parameters!ver_no = rSteps!version_no
    
            Set rIterators = qyIterators.OpenRecordset(dbOpenSnapshot)
            
            If rIterators.RecordCount <> 0 Then
                rIterators.MoveFirst
                While Not rIterators.EOF
                    Call InsertIterator(rIterators, cIds)
                    rIterators.MoveNext
                Wend
            End If
            rIterators.Close
        End If
        rSteps.MoveNext
    Wend
    
    Exit Sub
    
    qyIterators.Close
    qyConstraints.Close
    
    rSteps.Close
    qy.Close
    
    Set cIds = Nothing
    
    Exit Sub

LoadStepsErr:
    LogErrors Errors
    Err.Raise vbObjectError + errInsertFailed, mstrModuleName & "LoadSteps", _
            LoadResString(errInsertFailed)

End Sub
Private Sub LoadWspData(lWspId As Long)

    Dim sTemp As String
    Dim rTemp As DAO.Recordset
    Dim cTempSeq As cSequence
    Dim lNewWspId As Long
    Dim lId As Long
    
    On Error GoTo LoadWspDataErr
    
    Set cTempSeq = New cSequence
    Set cTempSeq.IdDatabase = dbNew
    
    cTempSeq.IdentifierColumn = FLD_ID_WORKSPACE
    
    sTemp = "select * from att_workspaces " & _
             "where workspace_id = " & CStr(lWspId)
    Set rTemp = dbOld.OpenRecordset(sTemp, dbOpenSnapshot)
    If rTemp.RecordCount <> 0 Then
        rTemp.MoveFirst
        
        While Not rTemp.EOF
            lNewWspId = cTempSeq.Identifier
            Call InsertWorkspace(rTemp, lNewWspId)
            rTemp.MoveNext
        Wend
    End If
    rTemp.Close
    
    Call LoadSteps(lWspId, lNewWspId)
    
    sTemp = "select * from workspace_parameters " & _
             "where workspace_id = " & CStr(lWspId)
    Set rTemp = dbOld.OpenRecordset(sTemp, dbOpenSnapshot)
    If rTemp.RecordCount <> 0 Then
        rTemp.MoveFirst
        cTempSeq.IdentifierColumn = FLD_ID_PARAMETER
        
        While Not rTemp.EOF
            Call InsertParameter(rTemp, cTempSeq.Identifier, lNewWspId)
            rTemp.MoveNext
        Wend
    End If
    rTemp.Close
    
    sTemp = "select * from workspace_connections " & _
             "where workspace_id = " & CStr(lWspId)
    Set rTemp = dbOld.OpenRecordset(sTemp, dbOpenSnapshot)
    If rTemp.RecordCount <> 0 Then
        rTemp.MoveFirst
        cTempSeq.IdentifierColumn = "connection_id"
        
        While Not rTemp.EOF
            Call InsertConnection(rTemp, cTempSeq.Identifier, lNewWspId)
            rTemp.MoveNext
        Wend
    End If
    rTemp.Close
    
    sTemp = "select * from " & TBL_CONNECTION_DTLS & _
             " where " & FLD_ID_WORKSPACE & " = " & CStr(lWspId)
    Set rTemp = dbOld.OpenRecordset(sTemp, dbOpenSnapshot)
    If rTemp.RecordCount <> 0 Then
        rTemp.MoveFirst
        cTempSeq.IdentifierColumn = FLD_ID_CONN_NAME
        
        While Not rTemp.EOF
            Call InsertConnDtl(rTemp, cTempSeq.Identifier, lNewWspId)
            rTemp.MoveNext
        Wend
    End If
    rTemp.Close
    
    Set cTempSeq = Nothing
    
    Exit Sub

LoadWspDataErr:
    LogErrors Errors
    Err.Raise vbObjectError + errInsertFailed, mstrModuleName & "LoadWspData", _
            LoadResString(errInsertFailed)

End Sub
